#include 	<avr/io.h>
#include 	<avr/interrupt.h>
#include 	<stdio.h>
#include 	<stdarg.h>
#include 	<string.h>
#include 	<stdlib.h>
#include 	<math.h>

#include 	"mydefs.h"
#include 	"printf.h"
#include 	"serial.h"
//---------------------------------------------------------------------------------/
#include 	"max7219.h"

#include 	"i2c.h"
#include 	"ds1307.h"
#include 	"adc.h"
//---------------------------------------------------------------------------------/
#define 	KEY			PINB
#define 	S_SET		_BV(PB2)
#define 	S_HOUR		_BV(PB3)
#define 	S_MIN		_BV(PB4)
#define 	S_SEC		_BV(PB5)

#define 	DIM_UP		_BV(PB3)
#define 	DIM_DOWN	_BV(PB4)
//---------------------------------------------------------------------------------/
//---------------------------------------------------------------------------------/
char mem_led	[8];
char mem_clock	[8];
char dimmer    = 2;			//default 2;
//---------------------------------------------------------------------------------/
//---------------------------------------------------------------------------------/
uint8_t 	getByte,
			interrupt,
			flag;
//---------------------------------------------------------------------------------/
void Delay (int t)
{
	while (t--)
		_delay_ms (1);
}
//---------------------------------------------------------------------------------//
void send_to_led (void)
{
	int i;
	
	for (i = 1; i < 9; i++)
	{
		mxbyte   (i,mem_clock[i-1]);	//clock
		mxbyte 	 (i,mem_led  [i-1]);	//circle
		
		cs_ ();
	}
}
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
void dot (char sec)
{
	const char BIT_ON[]  = {0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};
	
	int bit,digit;
	
	if (sec > 0) sec -= 1;
	bit   = (int)((sec)/8);	
	digit = (int)((sec)%8);
	
	mem_led[digit] |= BIT_ON[bit];
}
//---------------------------------------------------------------------------------//
ISR(INT0_vect)
{
	flag 	|= DS1307_INT;		//Set Flag interrupt buffer after INT0 interrupt
}
//---------------------------------------------------------------------------------//	
void init_rtc (void)
{
	/* set the I2C bit rate generator to 100 kb/s */
	twi_init ();
	//Enable INT 0 interrupt
	
	//  write logical 1 to ISC bit0 & bit1 
	MCUCR  = (1 << ISC01) | (0 << ISC00); //Falling egde interrupt
	
	check_DS1307 (0);
}
//---------------------------------------------------------------------------------//
void convert (char type)
{
		mem_clock[0] = RTC.hour/10;
		mem_clock[1] = RTC.hour%10;
			
		mem_clock[2] = RTC.min/10;
		mem_clock[3] = RTC.min%10;
	
		mem_clock[4] = RTC.sec/10;
		mem_clock[5] = RTC.sec%10;
		
		switch (type)
		{
			case 0 :	mem_clock[1] |= 0x80;	break;
			case 1 :	mem_clock[2] |= 0x80;	break;		
				
			case 2:		mem_clock[1] |= 0x80;
						mem_clock[2] |= 0x80;	break;
						
			default :	break;			
		}
		
	
}
//---------------------------------------------------------------------------------//
char kbd_getc (void)
{

	char 	nret = 0,
			nexit = 1;
	char    stChange = 0;
	
	
		Read_RTC ();
		
		while (nexit)
		{			
			if (!(KEY & S_HOUR))
			{
				Delay (0x0ff);
				if (RTC.hour++ >= 23) 	RTC.hour = 0;
				stChange = 1;
			}
			else
			{
				if (!(KEY & S_MIN))
				{
					Delay (0x0ff);
					if (RTC.min++ >= 59) 	RTC.min = 0;
					stChange = 1;
				}
				else
				{
					if (!(KEY & S_SEC))
					{
						Delay (0x0ff);
						if (RTC.sec++ >= 23) 	RTC.sec = 0;
						stChange = 1;
					}
					else
					{
						Delay (0x0ff);
						if (!(KEY & S_SET))
						{
							Delay (0x0ff);
							while (!(KEY & S_SET));	nexit = 0;
							new_update ();
								enable_rtc ();
							_delay_ms (100);
						}
					}
					
				}
			}
			if (stChange)
			{
				stChange = 0;
				convert (2);
				send_to_led ();
			}
		}
		//
	return nret;
}
//---------------------------------------------------------------------------------//
void dim (char dim)
{
	mxbyte (0x0A, dim);
	mxbyte (0x0A, dim);	//led bri
	cs_ ();
}
//---------------------------------------------------------------------------------//
void init_ntc (void)
{
	DDRC = 0xFE;			//Bit 0 are input temp
	PORTC |= _BV(0);
	init_adc ();						
}
//---------------------------------------------------------------------------------//
int main(void)
{
			
	uint16_t 	tmperature = 0,
				temp_H,
				temp_L;
	char 		stChange = 0,
				dimmer 	 = NORMAL_BRIGHT;
	
	ser_init();
	

	sei ();
	
	interrupt = 0;

	memset (mem_led, 	0, sizeof mem_led);
	memset (mem_clock, 	8, sizeof mem_clock);
	
	DDRB	= 0x00;
	PORTB  |= 0xFF;
	
	DDRD 	|= _BV(5);
	MAX_DDR |= _BV(DAT) | _BV(LOAD) | _BV(CLK);
	
	init_7219 ();
	
	init_ntc ();
	
	init_rtc ();
	
	enable_rtc ();
	
	send_to_led ();
	
	RTC.sec = 0;
	
	stChange = 0;
	
	while (1)
	{
		if (!(KEY & S_SET))
		{
			Delay (0x1ff);
				disable_rtc ();
			convert (2);
			send_to_led ();	
			while (!(KEY & S_SET));
				kbd_getc ();
			flag = 0;
			
			
		}
		else
		{
			//Dimmer brigness
			if (!(KEY & DIM_UP))
			{
				Delay (0x0ff);
				if (dimmer <= 0x0F) 
				{
					dimmer++;
					dim (dimmer);
				}
			}
			else
			{
				if(!(KEY & DIM_DOWN))
				{
					Delay (0x0ff);
					if (dimmer > 1) 
					{
						dimmer--;
						dim (dimmer);
					}
				}
			}
		};
		
		if (flag & DS1307_INT)
		{
			Read_RTC ();
			
			if (!(RTC.sec%15))tmperature = read_Temp ();
			
			if (RTC.sec%2)		convert (0);
			else				convert (1);			
						
				dot (RTC.sec);
			
			if (RTC.sec == 0) 	memset (mem_led, 0, sizeof mem_led);

			mem_led[3] |= 0x80;
			//mem_led[4] |= 0x20;
			//mem_led[5] |= 0x08; 
			//mem_led[6] |= 0x02;
			
			if ((RTC.sec > 5) && (RTC.sec < 10))
			{
				
				temp_H     = tmperature/10;	
				temp_L	   = tmperature%10;
				
				mem_clock[0] = temp_H/10;
				mem_clock[1] = temp_H%10;
				mem_clock[1] |= 0x80;
				mem_clock[2] = temp_L%10;
				
				mem_clock[3] = BLANK;
				mem_clock[4] = BLANK;
				mem_clock[5] = BLANK;
				
			}
			send_to_led ();
			flag = 0;
			
			if (((RTC.hour >= 18) || (RTC.hour <= 6)) && (!stChange)) // it mean dimer 18:00 to 06:00 next day
			{
				
				dim (LOW_BRIGHT);			//Low Bright
				stChange = 1;
			}
			else
			{
				if ((RTC.hour > 6) && (stChange))
				{
					dim (NORMAL_BRIGHT);	//Normal Bright
					stChange = 0;
				}
			}
		}
		
		_delay_ms (1);
	}
	
	

	while (1);
}
